% --- Source file: man/post_badge.Rd ---
\name{post_badge}
\alias{post_badge}
\title{Post processing badge output}
\description{
A function to summarize the output from the badge function
}
\usage{
post_badge(geno.mat, data, cc.var, exposure.var, group.var, out.dir, op=NULL)
}
\arguments{
  \item{geno.mat}{Matrix of genotype data. Genotype should be coded as 0, 1, or 2 for the number of copies of
                  the minor allele. The dimension of this matrix must be the number of subjects by the 
                  number of SNPs.}
  \item{data}{Data frame containing the disease status, exposure variable, group variable and possibly covariates.}
  \item{cc.var}{Variable name for the disease status. This variable should be coded as 0 for no disease 
               and 1 for disease.}
  \item{exposure.var}{Variable name for the exposure. This variable should be numerically coded.}
  \item{group.var}{Variable name for the groups. This variable should be coded as integers from 1 to the number of groups.}
  \item{out.dir}{Directory where the output files will be written.}
  \item{op}{List of options. See \code{details} for all possible options.}
}
\value{
  The returned value is list containing the deviance information criteria (\code{dic}),
  the cluster assignment for each subject (\code{subj.assign}),
  the first 5 principal components (\code{pc.mat}), 
  the median odds(alpha) for each subject (\code{alpha.med.odds}), and
  the median odds(beta) for each subject (\code{beta.med.odds})
  An output file of plots is written to \code{out.dir}.
}
\details{
  \code{geno.mat} and \code{data} should be the same objects that were used in \code{\link{define.NB.geno}} and
    \code{\link{badge}}.

 \bold{Options list:} \cr

 Below are the names for the options list \code{op}. All names have default values
         if they are not specified.
  \itemize{
  \item \code{covars} A character vector of variable names to be used as covariates.
                      Example: \code{covars=c("x1", "x2", "x3")}.
                      The default is that no covariates will be used in the analysis. 
  \item \code{out.string} Character string to be appended to the output file names. The default is "". 
  \item \code{M1} Starting iteration to use. The default is 1.
  \item \code{M2} Final iteration to use. The default is Inf.
  \item \code{everyN} Integer to use every \code{everyN} iterations. The default is 1.
  } % END: itemize

  \bold{Output file:} \cr

 The output file will contain 3 plots:
  \itemize{
  \item \code{1} Plot of the first 2 principal components
  \item \code{2} Heat map of the median odds(alpha) parameters
  \item \code{2} Heat map of the median odds(beta) parameters
  } % END: itemize


}
\seealso{\code{\link{define.NB.geno}} \code{\link{badge}}}
\author{Kai Yu}
