
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>

#include <R.h>
#include <R_ext/Memory.h>
#include <Rmath.h>
#include <Rinternals.h>
#include <R_ext/Rdynload.h>

#define MAXPARMLEN 5000
#define MAXLINELEN 100000
#define DOUBLE_TOL 1e-5
#define CHECK_MEM(obj) if (obj == NULL) {printf("ERROR: allocating memory \n"); exit(0);}
#define MAX( a, b ) ( ((a) > (b)) ? (a) : (b) )
#define MIN( a, b ) ( ((a) < (b)) ? (a) : (b) )
#define ZERO_THRS 1e-8
#define END_OF_LINE '\n'
#define CAR_RETURN '\r'
#define END_OF_STR '\0'
#define LOG_POINT5 -0.693147180559945
#define MAXPATHLEN 1000


struct data {
  int num_of_subj;
  int num_of_covars;
  int num_of_grp;
  int *grp_id;
  int *n_subj_grp;
  int **subj_grp;
  int *n_case_grp;
  int *n_control_grp;
  int *n_case_cluster;
  int *n_control_cluster;
  int *n_grp_cluster;

  int *disease_status;
  double *x;
  double *offset; /*for covarates */
  double **simi_mat;
  int **auxillary_sample;
  double alpha_min;
  double alpha_max;
  double alpha_mu;
  double alpha_sigma2;
  double beta_min;
  double beta_max;
  double beta_mu;
  double beta_sigma2;

  double sigma2_alpha;
  double sigma2_beta;
  double psi_k;
  double psi_theta;
  double psi_min;
  double psi_max;

  double *tao_min; /* for covariates */
  double *tao_max;
  double *tao_mu;
  double *tao_sigma2;
  int tao_min_n0, tao_max_n0, tao_mu_n0, tao_sigma2_n0;

  double sigma2_tao; 
  int method_psi; /* 0 uniform, 1 gamma, 2 call update_psi_discrete */
  double *psi_grid;
  double *prob_psi_grid;
  double *normalizing_psi_grid;
  int psi_position;
  int n_psi_grid;
  int method_alpha; /* 0 uniform, 1 normal */
  int method_beta; /* 0 uniform, 1 normal */
  int method_tao; /* 0 uniform, 1 normal */

  double sigma2_psi;
  int w_m;  /* number of vectors in the auxillary sample w */
  double update_prop;
  double update_prop_w;
  int n_iter;
  int n_sep_out;
  int random_seed;
  int *w0;  /* the initial auxillary vector */
  int k_max; /* the maxmium number of cluster in potts model */
  double **covars;

  /* Constants b, c, d for splitting/merging clusters */
  double *b;
  double *c;
  double *d;
  int split_option;  /* 0 = adaptive, 1 = fixed */
  int shape_split_a;
  int shape_split_b;
  int shape_merge_a;
  int shape_merge_b;

  /* the following values will be changed at each iteration */
  int num_of_cluster; /* defined as k */
  double psi_interaction;
  double *alpha;
  double *beta;
  double *tao; /* the length is the number of covariates */

  double dev_i;
  double cluster_size;  /*cluser size > it, or 0 */
  double prop_cut;    /* limit of prop of case/control in the cluster */
  int *alloc_z; /* taking value in 0:num_of_cluster -1 */

  int **neighbor_id; /* Matrix of neighbor ids */
  int *num_neighbor; /* Vector for the number of neighbors */ 
};

void print_iVec(int *, int, char *);
void print_dVec(double *, int, char *);

void quicksort(int, int, double *, int *);
void get_ranks(double *, int, int*);
int ** intMatrix_alloc(int, int);
void intMatrix_free(int **, int);
double ** dblMatrix_alloc(int, int);
void dblMatrix_free(double **, int);
int pri_gen_num_of_cluster(int);
double pri_pd_num_of_cluster(int, int);
double pri_gen_psi_unif(double, double);
double pri_pd_psi_unif(double, double, double);
double pri_gen_psi_gamma(double, double);
double pri_pd_psi_gamma(double, double, double);

void pri_gen_alpha(double *, int, double, double, int, double, double);
double pri_pd_alpha(double, double, double, int, double, double);
void pri_gen_beta(double *, int, double, double, int, double, double);
double pri_pd_beta(double, double, double, int, double, double);
void pri_gen_tao(double *, int, double *, double *, int, double *, double *);
double pri_pd_tao(double, double, double, int, double, double);
void ini_gen_alloc(int *, int, int);
double calc_Uz(double **, int *, int, int **, int *);

FILE * open_r(const char *);
int getNcols(const char *);
int getNrows(const char *);
void init(struct data *, const char *, const char *);
void free_memory(struct data *);
double rnormal(double, double);
void rnormal_vec(double, double, double *, int);
int bernoulli(double);
int sample1(double *, int);
double uniform(double, double);
void vecnorm(double *, double *, int);
double vecsum(double *, int);
void vecnormSum(double *, double *, int);
int checkVecBounds(double *, int, double, double);
void Vec1_p_cVec2(double *, double *, double *, int, double);
void vecinit(double *, int, double);
void ivecinit(int *, int, int);
void ivec_ivec_init(int *, int *, int);
double cal_deviance(int *, double *, int *, int, int *, double *, double *);
void log_P_alpha_beta(struct data *, double *, double *, double *);
void log_P_alpha_beta_tao(struct data *, double *, double *, 
			  double *, double *, double *);

void getVecStar(double *, double *, int, double);
int update_alpha_beta(struct data *);
int update_alpha_beta_tao(struct data *);
void compute_ti(double *, double *, int *, int, int, int *, int);
double compute_tij(double *, int *, int, int, int *, int);
double P_z_j(int *, double *, double *, int *, int,  double, double, double, double);

void cluster_counts(int *, int, int *, int);
void cluster_counts_case(int *, int, int *, int, int *, int);
void cluster_counts_weight(int *, int, int*, int, int *);

int accept_or_not_grp(int *,  int,  double);

int accept_or_not(int *, int *, int, double, double);
int update_1w(struct data *, int *, int *);
int update_all_w(struct data *, int *, int **);

int update_z(struct data *);
void gen_alloc_cond1(int *, double, int, double **, int*, int, int **, int *);
void gen_alloc_cond(int **, double, int, double **, int*, int, int, int **, int *);
int update_psi(struct data *);
int update_psi_discrete(struct data *);

void check_z(struct data *);
double fixed_prob(struct data *, double, double, double, double, int);
double adapt_prob(struct data *, double, double, double, double, int, int *, int, int);
double assign_split(struct data *, double, double, double, double, int *, int, int, int);
double assign_merge(struct data *, double, double, double, double, int *, int, int, int);
int rank_index(int *, int, int);
double log_prob_1clus(struct data *, double, double, int *, int);
void get_betaj12_merge(double *, int *, int, int, int,
                       double *, double *, double, double);

int fun_merge(struct data *);

void get_betaj12_split(double *, int *, int, int,
                       double *, double *, double, double);

int fun_split(struct data *);

double gen_beta(double, double, double, double);
double prob_beta(double, double, double, double, double);
double logprob_calculate(double, int);
double logR_to_R(double);
int uniform_int(int, int);


